/*
 * ====================================================================
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.http.client.cache;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Used by some {@link HttpCacheStorage} implementations to serialize
 * {@link HttpCacheEntry} instances to a byte representation before
 * storage.
 */
public interface HttpCacheEntrySerializer {

    /**
     * Serializes the given entry to a byte representation on the
     * given {@link OutputStream}.
     * @throws IOException
     */
    void writeTo(HttpCacheEntry entry, OutputStream os) throws IOException;

    /**
     * Deserializes a byte representation of a cache entry by reading
     * from the given {@link InputStream}.
     * @throws IOException
     */
    HttpCacheEntry readFrom(InputStream is) throws IOException;

}
